@@ -484,10 +484,12 @@ gpb.cv <- function(params = list()
         cov_fct_shape <- gp_model$get_cov_fct_shape()
         cov_fct_taper_range <- gp_model$get_cov_fct_taper_range()
         ind_effect_group_rand_coef <- gp_model$get_ind_effect_group_rand_coef()
+        drop_intercept_group_rand_effect <- gp_model$get_drop_intercept_group_rand_effect()
         
         gp_model_train <- gpb.GPModel$new(group_data = group_data,
                                           group_rand_coef_data = group_rand_coef_data,
                                           ind_effect_group_rand_coef = ind_effect_group_rand_coef,
+                                          drop_intercept_group_rand_effect = drop_intercept_group_rand_effect,
                                           gp_coords = gp_coords,
                                           gp_rand_coef_data = gp_rand_coef_data,
                                           cov_function = cov_function,
@@ -522,7 +524,6 @@ gpb.cv <- function(params = list()
         booster <- Booster$new(params = params, train_set = dtrain, gp_model = gp_model_train)
         gp_model$set_likelihood(gp_model_train$get_likelihood_name()) ## potentially change likelihood in case this was done in the booster to reflect implied changes in the default optimizer for different likelihoods
         gp_model_train$set_optim_params(params = gp_model$get_optim_params())
-        gp_model_train$set_optim_coef_params(params = gp_model$get_optim_params())
         
       } else {
         booster <- Booster$new(params = params, train_set = dtrain)
