@@ -811,6 +811,7 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 	SEXP re_group_rand_coef_data,
 	SEXP ind_effect_group_rand_coef,
 	SEXP num_re_group_rand_coef,
+	SEXP drop_intercept_group_rand_effect,
 	SEXP num_gp,
 	SEXP gp_coords_data,
 	SEXP dim_gp_coords,
@@ -850,6 +851,7 @@ SEXP GPB_CreateREModel_R(SEXP ndata,
 		R_REAL_PTR(re_group_rand_coef_data),
 		ind_eff_group_rand_coef,
 		num_regroup_rand_coef,
+		R_INT_PTR(drop_intercept_group_rand_effect),
 		numgp,
 		R_REAL_PTR(gp_coords_data),
 		Rf_asInteger(dim_gp_coords),
@@ -965,11 +967,22 @@ SEXP GPB_OptimLinRegrCoefCovPar_R(SEXP handle,
 SEXP GPB_EvalNegLogLikelihood_R(SEXP handle,
 	SEXP y_data,
 	SEXP cov_pars,
+	SEXP fixed_effects,
 	SEXP negll) {
 	R_API_BEGIN();
 	CHECK_CALL(GPB_EvalNegLogLikelihood(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(y_data),
 		R_REAL_PTR(cov_pars),
+		R_REAL_PTR(fixed_effects),
+		R_REAL_PTR(negll)));
+	R_API_END();
+	return R_NilValue;
+}
+
+SEXP GPB_GetCurrentNegLogLikelihood_R(SEXP handle,
+	SEXP negll) {
+	R_API_BEGIN();
+	CHECK_CALL(GPB_GetCurrentNegLogLikelihood(R_ExternalPtrAddr(handle),
 		R_REAL_PTR(negll)));
 	R_API_END();
 	return R_NilValue;
@@ -1218,13 +1231,14 @@ static const R_CallMethodDef CallEntries[] = {
   {"LGBM_BoosterSaveModel_R"          , (DL_FUNC)&LGBM_BoosterSaveModel_R          , 4},
   {"LGBM_BoosterSaveModelToString_R"  , (DL_FUNC)&LGBM_BoosterSaveModelToString_R  , 4},
   {"LGBM_BoosterDumpModel_R"          , (DL_FUNC)&LGBM_BoosterDumpModel_R          , 3},
-  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 21},
+  {"GPB_CreateREModel_R"              , (DL_FUNC)&GPB_CreateREModel_R              , 22},
   {"GPB_REModelFree_R"                , (DL_FUNC)&GPB_REModelFree_R                , 1},
   {"GPB_SetOptimConfig_R"             , (DL_FUNC)&GPB_SetOptimConfig_R             , 13},
   {"GPB_SetOptimCoefConfig_R"         , (DL_FUNC)&GPB_SetOptimCoefConfig_R         , 6},
   {"GPB_OptimCovPar_R"                , (DL_FUNC)&GPB_OptimCovPar_R                , 3},
   {"GPB_OptimLinRegrCoefCovPar_R"     , (DL_FUNC)&GPB_OptimLinRegrCoefCovPar_R     , 4},
-  {"GPB_EvalNegLogLikelihood_R"       , (DL_FUNC)&GPB_EvalNegLogLikelihood_R       , 4},
+  {"GPB_EvalNegLogLikelihood_R"       , (DL_FUNC)&GPB_EvalNegLogLikelihood_R       , 5},
+  {"GPB_GetCurrentNegLogLikelihood_R" , (DL_FUNC)&GPB_GetCurrentNegLogLikelihood_R , 2},
   {"GPB_GetCovPar_R"                  , (DL_FUNC)&GPB_GetCovPar_R                  , 3},
   {"GPB_GetInitCovPar_R"              , (DL_FUNC)&GPB_GetInitCovPar_R              , 2},
   {"GPB_GetCoef_R"                    , (DL_FUNC)&GPB_GetCoef_R                    , 3},
