@@ -619,6 +619,7 @@ GPBOOST_C_EXPORT SEXP LGBM_BoosterDumpModel_R(
 * \param re_group_rand_coef_data Covariate data for grouped random coefficients
 * \param ind_effect_group_rand_coef Indices that relate every random coefficients to a "base" intercept grouped random effect. Counting starts at 1.
 * \param num_re_group_rand_coef Number of grouped random coefficients
+* \param drop_intercept_group_rand_effect Indicates whether intercept random effects are dropped (only for random coefficients). If drop_intercept_group_rand_effect[k] > 0, the intercept random effect number k is dropped. Only random effects with random slopes can be dropped.
 * \param num_gp Number of Gaussian processes (intercept only, random coefficients not counting)
 * \param gp_coords_data Coordinates (features) for Gaussian process
 * \param dim_gp_coords Dimension of the coordinates (=number of features) for Gaussian process
@@ -643,6 +644,7 @@ GPBOOST_C_EXPORT SEXP GPB_CreateREModel_R(
 	SEXP re_group_rand_coef_data,
 	SEXP ind_effect_group_rand_coef,
 	SEXP num_re_group_rand_coef,
+	SEXP drop_intercept_group_rand_effect,
 	SEXP num_gp,
 	SEXP gp_coords_data,
 	SEXP dim_gp_coords,
@@ -754,6 +756,7 @@ GPBOOST_C_EXPORT SEXP GPB_OptimLinRegrCoefCovPar_R(
 * \param handle Handle of REModel
 * \param y_data Response variable data
 * \param cov_pars Values for covariance parameters of RE components
+* \param fixed_effects Fixed effects component of location parameter for observed data (only used for non-Gaussian data). For Gaussian data, this is ignored
 * \param[out] negll Negative log-likelihood
 * \return 0 when succeed, -1 when failure happens
 */
@@ -761,6 +764,18 @@ GPBOOST_C_EXPORT SEXP GPB_EvalNegLogLikelihood_R(
 	SEXP handle,
 	SEXP y_data,
 	SEXP cov_pars,
+	SEXP fixed_effects,
+	SEXP negll
+);
+
+/*!
+* \brief Get the current value of the negative log-likelihood
+* \param handle Handle of REModel
+* \param[out] negll Negative log-likelihood
+* \return 0 when succeed, -1 when failure happens
+*/
+GPBOOST_C_EXPORT SEXP GPB_GetCurrentNegLogLikelihood_R(
+	SEXP handle,
 	SEXP negll
 );
 
