@@ -1327,6 +1327,7 @@ GPBOOST_C_EXPORT int LGBM_NetworkInitWithFunctions(int num_machines,
 * \param re_group_rand_coef_data Covariate data for grouped random coefficients
 * \param ind_effect_group_rand_coef Indices that relate every random coefficients to a "base" intercept grouped random effect. Counting starts at 1.
 * \param num_re_group_rand_coef Number of grouped random coefficients
+* \param drop_intercept_group_rand_effect Indicates whether intercept random effects are dropped (only for random coefficients). If drop_intercept_group_rand_effect[k] > 0, the intercept random effect number k is dropped. Only random effects with random slopes can be dropped.
 * \param num_gp Number of Gaussian processes (intercept only, random coefficients not counting)
 * \param gp_coords_data Coordinates (features) for Gaussian process
 * \param dim_gp_coords Dimension of the coordinates (=number of features) for Gaussian process
@@ -1351,6 +1352,7 @@ GPBOOST_C_EXPORT int GPB_CreateREModel(int32_t num_data,
     const double* re_group_rand_coef_data,
     const int32_t* ind_effect_group_rand_coef,
     int32_t num_re_group_rand_coef,
+    const int* drop_intercept_group_rand_effect,
     int32_t num_gp,
     const double* gp_coords_data,
     const int dim_gp_coords,
@@ -1426,7 +1428,7 @@ GPBOOST_C_EXPORT int GPB_SetOptimCoefConfig(REModelHandle handle,
 * \brief Find parameters that minimize the negative log-ligelihood (=MLE)
 * \param handle Handle of REModel
 * \param y_data Response variable data
-* \param fixed_effects Fixed effects component F of location parameter (only used for non-Gaussian data). For Gaussian data, this is ignored
+* \param fixed_effects Fixed effects component of location parameter (only used for non-Gaussian data). For Gaussian data, this is ignored
 * \return 0 when succeed, -1 when failure happens
 */
 GPBOOST_C_EXPORT int GPB_OptimCovPar(REModelHandle handle,
@@ -1452,12 +1454,23 @@ GPBOOST_C_EXPORT int GPB_OptimLinRegrCoefCovPar(REModelHandle handle,
 * \param handle Handle of REModel
 * \param y_data Response variable data
 * \param cov_pars Values for covariance parameters of RE components
+* \param fixed_effects Fixed effects component of location parameter for observed data (only used for non-Gaussian data). For Gaussian data, this is ignored
 * \param[out] negll Negative log-likelihood
 * \return 0 when succeed, -1 when failure happens
 */
 GPBOOST_C_EXPORT int GPB_EvalNegLogLikelihood(REModelHandle handle,
     const double* y_data,
     double* cov_pars,
+    const double* fixed_effects,
+    double* negll);
+
+/*!
+* \brief Get the current value of the negative log-likelihood
+* \param handle Handle of REModel
+* \param[out] negll Negative log-likelihood
+* \return 0 when succeed, -1 when failure happens
+*/
+GPBOOST_C_EXPORT int GPB_GetCurrentNegLogLikelihood(REModelHandle handle,
     double* negll);
 
 /*!
@@ -1547,7 +1560,7 @@ GPBOOST_C_EXPORT int GPB_SetPredictionData(REModelHandle handle,
 * \param use_saved_data If true previusly set data on groups, coordinates, and covariates are used and some arguments of this function are ignored
 * \param vecchia_pred_type Type of Vecchia approximation for making predictions. "order_obs_first_cond_obs_only" = observed data is ordered first and neighbors are only observed points, "order_obs_first_cond_all" = observed data is ordered first and neighbors are selected among all points (observed + predicted), "order_pred_first" = predicted data is ordered first for making predictions, "latent_order_obs_first_cond_obs_only"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are only observed points, "latent_order_obs_first_cond_all"  = Vecchia approximation for the latent process and observed data is ordered first and neighbors are selected among all points
 * \param num_neighbors_pred The number of neighbors used in the Vecchia approximation for making predictions (-1 means that the value already set at initialization is used)
-* \param fixed_effects Fixed effects component of location parameter for observed data (only used for non-Gaussian data)
+* \param fixed_effects Fixed effects component of location parameter for observed data (only used for non-Gaussian data). For Gaussian data, this is ignored
 * \param fixed_effects_pred Fixed effects component of location parameter for predicted data (only used for non-Gaussian data)
 * \return 0 when succeed, -1 when failure happens
 */
@@ -1577,7 +1590,7 @@ GPBOOST_C_EXPORT int GPB_PredictREModel(REModelHandle handle,
 * \param cov_pars_pred Covariance parameters of components
 * \param y_obs Response variable for observed data
 * \param[out] out_predict Predicted training data random effects
-* \param fixed_effects Fixed effects component of location parameter for observed data (only used for non-Gaussian data)
+* \param fixed_effects Fixed effects component of location parameter for observed data (only used for non-Gaussian data). For Gaussian data, this is ignored
 * \return 0 when succeed, -1 when failure happens
 */
 GPBOOST_C_EXPORT int GPB_PredictREModelTrainingDataRandomEffects(REModelHandle handle,
