@@ -1,5 +1,10 @@
 context("GPModel_grouped_random_effects")
 
+TOLERANCE <- 1E-3
+TOLERANCE2 <- 1E-2
+TOLERANCE1 <- 1E-1
+TOL_STRICT <- 1E-6
+
 if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
   
   # Function that simulates uniform random variables
@@ -45,7 +50,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     fit(gp_model, y = y, params = list(std_dev = TRUE, optimizer_cov = "fisher_scoring",
                                        convergence_criterion = "relative_change_in_parameters"))
     cov_pars <- c(0.49348532, 0.02326312, 1.22299521, 0.17995161)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
     expect_equal(dim(gp_model$get_cov_pars())[2], 2)
     expect_equal(dim(gp_model$get_cov_pars())[1], 2)
     expect_equal(gp_model$get_num_optim_iter(), 6)
@@ -72,19 +77,24 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE,
                                          lr_cov = 10, use_nesterov_acc = FALSE,
                                          maxit = 1000, convergence_criterion = "relative_change_in_parameters"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 32)
     # Different termination criterion
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_log_likelihood"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 5)
     # Nelder-Mead
     gp_model <- fitGPModel(group_data = group, y = y,
                            params = list(optimizer_cov = "nelder_mead", std_dev = TRUE))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-3)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE)
     expect_equal(gp_model$get_num_optim_iter(), 12)
+    # Adam
+    gp_model <- fitGPModel(group_data = group, y = y,
+                           params = list(optimizer_cov = "adam", std_dev = TRUE))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE)
+    expect_equal(gp_model$get_num_optim_iter(), 275)
     
     # Evaluation of likelihood
     ll <- gp_model$neg_log_likelihood(y=y,cov_pars=gp_model$get_cov_pars()[1,])
@@ -99,13 +109,13 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(-0.1553877, -0.3945731, 0)
     expected_cov <- c(0.5483871, 0.0000000, 0.0000000, 0.0000000,
                       0.5483871, 0.0000000, 0.0000000, 0.0000000, 2)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
     # Predict variances
     pred <- predict(gp_model, y=y, group_data_pred = group_test,
                     cov_pars = c(0.5,1.5), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOL_STRICT)
     
     # Prediction from fitted model
     gp_model <- fitGPModel(group_data = group, y = y,
@@ -116,8 +126,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(-0.1543396, -0.3919117, 0.0000000)
     expected_cov <- c(0.5409198 , 0.0000000000, 0.0000000000, 0.0000000000,
                       0.5409198 , 0.0000000000, 0.0000000000, 0.0000000000, 1.7164805)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
     
     # Predict training data random effects
     all_training_data_random_effects <- predict_training_data_random_effects(gp_model)
@@ -125,7 +135,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     training_data_random_effects <- all_training_data_random_effects[first_occurences] 
     group_unique <- unique(group)
     pred_random_effects <- predict(gp_model, group_data_pred = group_unique)
-    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),1E-6)
+    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOL_STRICT)
     
     # Evaluate negative log-likelihood
     nll <- gp_model$neg_log_likelihood(cov_pars=c(0.1,1),y=y)
@@ -134,7 +144,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     # Do optimization using optim and e.g. Nelder-Mead
     gp_model <- GPModel(group_data = group)
     opt <- optim(par=c(1,1), fn=gp_model$neg_log_likelihood, y=y, method="Nelder-Mead")
-    expect_lt(sum(abs(opt$par-cov_pars[c(1,3)])),1E-3)
+    expect_lt(sum(abs(opt$par-cov_pars[c(1,3)])),TOLERANCE)
     expect_lt(abs(opt$value-(1228.293)),1E-2)
     expect_equal(as.integer(opt$counts[1]), 49)
     
@@ -144,7 +154,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     gp_model <- GPModel(group_data = group[shuffle_ind])
     fit(gp_model, y = y[shuffle_ind], params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE,
                                                     convergence_criterion = "relative_change_in_parameters"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 6)
     
     # Inf in response variable data gives error
@@ -165,8 +175,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          convergence_criterion = "relative_change_in_parameters"))
     cov_pars <- c(0.49205230, 0.02319557, 1.22064076, 0.17959832)
     coef <- c(2.07499902, 0.11269252, 1.94766255, 0.03382472)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 7)
     
     # Prediction
@@ -178,8 +188,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.886494, 2.043259, 2.854064)
     expected_cov <- c(0.5393509 , 0.0000000000, 0.0000000000, 0.0000000000,
                       0.5393509 , 0.0000000000, 0.0000000000, 0.0000000000, 1.712693)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
     
     # Predict training data random effects
     all_training_data_random_effects <- predict_training_data_random_effects(gp_model)
@@ -188,38 +198,39 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_unique <- unique(group)
     X_zero <- cbind(rep(0,length(group_unique)),rep(0,length(group_unique)))
     pred_random_effects <- predict(gp_model, group_data_pred = group_unique, X_pred = X_zero)
-    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),1E-6)
+    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOL_STRICT)
     
     # Fit model using gradient descent instead of wls for regression coefficients
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "gradient_descent", maxit=1000, std_dev = TRUE,
                                          optimizer_coef = "gradient_descent", lr_coef=1, use_nesterov_acc=TRUE))
-    cov_pars <- c(0.49213311, 0.02319938, 1.22051070, 0.17958108)
-    coef <- c(2.07485695, 0.11268711, 1.95215530, 0.03382747)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOLERANCE2)
     expect_equal(gp_model$get_num_optim_iter(), 8)
     
     # Fit model using Nelder-Mead
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "nelder_mead",
-                                         optimizer_coef = "nelder_mead", std_dev = TRUE, delta_rel_conv=1e-6))
-    cov_pars <- c(0.49215382, 0.02320035, 1.21828714, 0.17926694)
-    coef <- c(2.07198836, 0.11258850, 1.94792785, 0.03382805)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
+                                         optimizer_coef = "nelder_mead", std_dev = FALSE, delta_rel_conv=1e-6))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOLERANCE2)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef[c(1,3)])),TOLERANCE2)
     expect_equal(gp_model$get_num_optim_iter(), 125)
+    # Fit model using Adam
+    gp_model <- fitGPModel(group_data = group,
+                           y = y, X = X,
+                           params = list(optimizer_cov = "adam", std_dev = FALSE, delta_rel_conv=1e-6))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars[c(1,3)])),TOLERANCE2)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef[c(1,3)])),TOLERANCE2)
+    expect_equal(gp_model$get_num_optim_iter(), 354)
     
     # Fit model using BFGS
     gp_model <- fitGPModel(group_data = group,
                            y = y, X = X,
                            params = list(optimizer_cov = "bfgs", std_dev = TRUE))
-    cov_pars <- c(0.49205229, 0.02319557, 1.22064060, 0.17959830)
-    coef <- c(2.07499895, 0.11269251, 1.94766254, 0.03382472)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 11)
     
     # Large data
@@ -240,8 +251,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          optimizer_coef = "gradient_descent", lr_coef=0.1, use_nesterov_acc=TRUE))
     cov_pars <- c(0.5005068978, 0.0007461116, 0.9982863693, 0.0046888995)
     coef <- c(1.994283503, 0.003484753, 2.004006668, 0.002577150)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 7)
     
     gp_model <- fitGPModel(group_data = group_L,
@@ -249,8 +260,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "nelder_mead", maxit=1000))
     cov_pars <- c(0.5004681, 0.9988288)
     coef <- c(2.000747, 1.999343)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(gp_model$get_coef())-coef)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 152)
     
   })
@@ -264,7 +275,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     gp_model <- fitGPModel(group_data = cbind(group,group2), y = y, 
                            params = list(optimizer_cov = "fisher_scoring", std_dev = TRUE))
     expected_values <- c(0.49792062, 0.02408196, 1.21972166, 0.18357646, 1.06962710, 0.22567292)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 5)
     
     # Prediction after training
@@ -272,7 +283,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred, predict_var = TRUE)
     expected_mu <- c(0.7509175, -0.4208015, 0.0000000)
     expected_var <- c(0.5677178, 0.5677178, 2.7872694)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
     expect_lt(sum(abs(as.vector(pred$var)-expected_var)),1E-4)
     # Prediction without training and parameters given
     gp_model <- GPModel(group_data = cbind(group,group2))
@@ -281,24 +292,24 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.7631462, -0.4328551, 0.000000000)
     expected_cov <- c(0.114393721, 0.009406189, 0.0000000, 0.009406189,
                       0.114393721 , 0.0000000, 0.0000000, 0.0000000, 3.100000000)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-3)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
     # Prediction for only existing random effects
     group_data_pred_in = cbind(c(1,1),c(2,1))
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred_in,
                              cov_pars = c(0.1,1,2), predict_cov_mat = TRUE)
     expected_mu <- c(0.7631462, -0.4328551)
     expected_cov <- c(0.114393721, 0.009406189, 0.009406189, 0.114393721)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-3)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
     # Prediction for only new random effects
     group_data_pred_out = cbind(c(m+1,m+1,m+1),c(length(group2)+1,length(group2)+2,length(group2)+1))
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred_out,
                              cov_pars = c(0.1,1,2), predict_cov_mat = TRUE)
     expected_mu <- c(rep(0,3))
     expected_cov <- c(3.1, 1.0, 3.0, 1.0, 3.1, 1.0, 3.0, 1.0, 3.1)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-3)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
     
     ## Two crossed random effects and a random slope
     y <- Z1%*%b1 + Z2%*%b2 + Z3%*%b3 + xi
@@ -308,7 +319,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "fisher_scoring", maxit=5, std_dev = TRUE))
     expected_values <- c(0.49554952, 0.02546769, 1.24880860, 0.18983953, 1.05505134, 0.22337199, 1.13840014, 0.17950490)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 5)
     
     # Predict training data random effects
@@ -322,17 +333,17 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_data_pred = cbind(group_unique,rep(-1,length(group_unique)))
     x_pr = rep(0,length(group_unique))
     preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr)
-    expect_lt(sum(abs(pred_random_effects - preds$mu)),1E-6)
+    expect_lt(sum(abs(pred_random_effects - preds$mu)),TOL_STRICT)
     # Check whether random slopes are correct
     x_pr = rep(1,length(group_unique))
     preds2 <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr)
-    expect_lt(sum(abs(pred_random_slopes - (preds2$mu-preds$mu))),1E-6)
+    expect_lt(sum(abs(pred_random_slopes - (preds2$mu-preds$mu))),TOL_STRICT)
     # Check whether crossed random effects are correct
     group_unique <- unique(group2)
     group_data_pred = cbind(rep(-1,length(group_unique)),group_unique)
     x_pr = rep(0,length(group_unique))
     preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr)
-    expect_lt(sum(abs(pred_random_effects_crossed - preds$mu)),1E-6)
+    expect_lt(sum(abs(pred_random_effects_crossed - preds$mu)),TOL_STRICT)
     
     # Prediction
     gp_model <- GPModel(group_data = cbind(group,group2),
@@ -347,14 +358,14 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(0.7579961, -0.2868530, 0.000000000)
     expected_cov <- c(0.11534086, -0.01988167, 0.0000000, -0.01988167, 2.4073302,
                       0.0000000, 0.0000000, 0.0000000, 3.235)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-3)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
     # Predict variances
     pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
                              group_rand_coef_data_pred=group_rand_coef_data_pred,
                              cov_pars = c(0.1,1,2,1.5), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),1E-3)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE)
     
     # Gradient descent
     gp_model <- fitGPModel(group_data = cbind(group,group2),
@@ -363,7 +374,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "gradient_descent", std_dev = FALSE))
     cov_pars <- c(0.4958303, 1.2493181, 1.0543343, 1.1388604 )
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(),8)
     
     # Nelder-Mead
@@ -372,23 +383,80 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            ind_effect_group_rand_coef = 1,
                            y = y,
                            params = list(optimizer_cov = "nelder_mead", std_dev = FALSE))
-    cov_pars <- c(0.4959521, 1.2408579, 1.0560989, 1.1373089 )
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
-    
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE1)
     # BFGS
     gp_model <- fitGPModel(group_data = cbind(group,group2),
                            group_rand_coef_data = x,
                            ind_effect_group_rand_coef = 1,
                            y = y,
                            params = list(optimizer_cov = "bfgs", std_dev = FALSE))
-    cov_pars <- c(0.4955502, 1.2488000, 1.0550351, 1.1383709 )
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE2)
+    # Adam
+    gp_model <- fitGPModel(group_data = cbind(group,group2),
+                           group_rand_coef_data = x,
+                           ind_effect_group_rand_coef = 1,
+                           y = y,
+                           params = list(optimizer_cov = "adam", std_dev = FALSE))
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars)),TOLERANCE2)
     
     # Evaluate negative log-likelihood
     nll <- gp_model$neg_log_likelihood(cov_pars=c(0.1,1,2,1.5),y=y)
     expect_lt(abs(nll-2335.803),1E-2)
   })
   
+  test_that("Random coefficients with intercept random effect dropped ", {
+    
+    ## A random effect and a random slope without a corresponding intercept effect
+    y <- Z2%*%b2 + Z3%*%b3 + xi
+    gp_model <- fitGPModel(group_data = cbind(group,group2),
+                           group_rand_coef_data = x, ind_effect_group_rand_coef = 1,
+                           drop_intercept_group_rand_effect = c(TRUE,FALSE), y = y,
+                           params = list(optimizer_cov = "gradient_descent"))
+    expected_values <- c(0.5017205, 1.0818474, 1.1157430)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
+    expect_equal(gp_model$get_num_optim_iter(), 7)
+    
+    # Predict training data random effects
+    all_training_data_random_effects <- predict_training_data_random_effects(gp_model)
+    first_occurences_1 <- match(unique(group), group)
+    first_occurences_2 <- match(unique(group2), group2)
+    pred_random_slopes <- all_training_data_random_effects[first_occurences_1,2]
+    pred_random_effects_crossed <- all_training_data_random_effects[first_occurences_2,1] 
+    group_unique <- unique(group)
+    group_data_pred = cbind(group_unique,rep(-1,length(group_unique)))
+    # Check whether random slopes are correct
+    x_pr = rep(1,length(group_unique))
+    preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr)
+    expect_lt(sum(abs(pred_random_slopes - preds$mu)),TOL_STRICT)
+    # Check whether crossed random effects are correct
+    group_unique <- unique(group2)
+    group_data_pred = cbind(rep(-1,length(group_unique)),group_unique)
+    x_pr = rep(0,length(group_unique))
+    preds <- predict(gp_model, group_data_pred=group_data_pred, group_rand_coef_data_pred=x_pr)
+    expect_lt(sum(abs(pred_random_effects_crossed - preds$mu)),TOL_STRICT)
+    
+    # Prediction
+    gp_model <- GPModel(group_data = cbind(group,group2),
+                        group_rand_coef_data = x, ind_effect_group_rand_coef = 1,
+                        drop_intercept_group_rand_effect = c(TRUE,FALSE))
+    group_data_pred = cbind(c(1,1,m+1),c(2,1,length(group2)+1))
+    group_rand_coef_data_pred = c(0,10,0.3)
+    pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
+                             group_rand_coef_data_pred=group_rand_coef_data_pred,
+                             cov_pars = c(0.1,2,1.5), predict_cov_mat = TRUE)
+    expected_mu <- c(0.8426751, -0.5964363, 0.000000000)
+    expected_cov <- c(0.10558205, -0.01269261, 0.00000000, -0.01269261, 2.40180871,
+                      0.00000000, 0.00000000, 0.00000000, 2.23500000)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOLERANCE)
+    # Predict variances
+    pred <- gp_model$predict(y = y, group_data_pred=group_data_pred,
+                             group_rand_coef_data_pred=group_rand_coef_data_pred,
+                             cov_pars = c(0.1,2,1.5), predict_var = TRUE)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_cov[c(1,5,9)])),TOLERANCE)
+    
+  })
   
   test_that("not constant cluster_id's for grouped random effects ", {
     
@@ -398,7 +466,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            params = list(optimizer_cov = "fisher_scoring", maxit=100, std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_parameters"))
     expected_values <- c(0.49348532, 0.02326312, 1.22299521, 0.17995161)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 6)
     
     # gradient descent
@@ -408,7 +476,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          lr_cov = 0.1, use_nesterov_acc = FALSE, maxit = 1000,
                                          convergence_criterion = "relative_change_in_parameters"))
     cov_pars_expected <- c(0.49348532, 0.02326312, 1.22299520, 0.17995161)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 9)
     
     # Predict training data random effects
@@ -418,7 +486,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     group_unique <- unique(group)
     pred_random_effects <- predict(gp_model, group_data_pred = group_unique, 
                                    cluster_ids_pred = cluster_ids[first_occurences])
-    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),1E-6)
+    expect_lt(sum(abs(training_data_random_effects - pred_random_effects$mu)),TOL_STRICT)
     
     # Prediction
     group_data_pred = c(1,1,m+1)
@@ -432,8 +500,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     expected_mu <- c(-0.1514786, 0.000000, 0.000000)
     expected_cov <- c(0.8207547, 0.000000, 0.000000, 0.000000,
                       2.000000, 0.000000, 0.000000, 0.000000, 2.000000)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
     
     # cluster_ids can be string 
     cluster_ids_string <- paste0(as.character(cluster_ids),"_s")
@@ -441,7 +509,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                            y = y,
                            params = list(optimizer_cov = "fisher_scoring", maxit=100, std_dev = TRUE,
                                          convergence_criterion = "relative_change_in_parameters"))
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-cov_pars_expected)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 6)
     # Prediction
     group_data_pred = c(1,1,m+1)
@@ -449,8 +517,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     pred <- gp_model$predict(y = y, group_data_pred = group_data_pred,
                              cluster_ids_pred = cluster_ids_pred_string,
                              cov_pars = c(0.75,1.25), predict_cov_mat = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$cov)-expected_cov)),TOL_STRICT)
     
     # Prediction when cluster_ids are not in ascending order
     group_data_pred = c(1,1,m,m)
@@ -461,8 +529,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                              cov_pars = c(0.75,1.25), predict_var = TRUE)
     expected_mu <- c(rep(0,3), 1.179557)
     expected_var <- c(rep(2,3), 0.8207547)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),TOL_STRICT)
     # same thing when cluster_ids are strings
     group_data_pred = c(1,1,m,m)
     cluster_ids_pred_string = paste0(as.character(c(2,2,1,2)),"_s")
@@ -470,8 +538,8 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
     pred <- gp_model$predict(y = y, group_data_pred = group_data_pred,
                              cluster_ids_pred = cluster_ids_pred_string,
                              cov_pars = c(0.75,1.25), predict_var = TRUE)
-    expect_lt(sum(abs(pred$mu-expected_mu)),1E-6)
-    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),1E-6)
+    expect_lt(sum(abs(pred$mu-expected_mu)),TOL_STRICT)
+    expect_lt(sum(abs(as.vector(pred$var)-expected_var)),TOL_STRICT)
 
     # cluster_ids and random coefficients
     y <- Z1%*%b1 + Z3%*%b3 + xi
@@ -484,7 +552,7 @@ if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
                                          lr_cov = 0.1, use_nesterov_acc = TRUE, maxit = 1000,
                                          convergence_criterion = "relative_change_in_parameters"))
     expected_values <- c(0.4927786, 1.2565095, 1.1333656)
-    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),1E-6)
+    expect_lt(sum(abs(as.vector(gp_model$get_cov_pars())-expected_values)),TOL_STRICT)
     expect_equal(gp_model$get_num_optim_iter(), 14)
     
   })
